EnchantList = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")
local L = AceLibrary("AceLocale-2.2"):new("EnchantList")
local chatprefix = "!enchant"
local LOCKED=0
local mem_search={}
local mem_next={}

function EnchantList:OnInitialize()
	self:Reset()
	EnchantList:RegisterDB("EnchantListDB")
	EnchantList:RegisterDefaults("account", {
		enchants = {},
		enchantsname= {},
		enchantsdesc={},
		ignoredenchants={},
		showincmessages=false,
		showexmessages=false,
		searchdesc=true,
		autoscan=true,
		TIMELOCK=10,
		chatprefix2="eList>",
		maxresults=10,
		gmaxresults=3,
		guildactive=true,
		}
	)  
	
local options = { 
    type="group",
    args = {
    	inc= {
    		type="toggle",
    		name=L["Show incoming requests"],
    		desc=L["Show incoming requests"],
    		get=function() return self.db.account.showincmessages end,
    		set=function(v) self.db.account.showincmessages = v end,
    		},
			out= {
				type="toggle",
				name=L["Show your replies"],
				desc=L["Show your replies"],
				get=function() return self.db.account.showexmessages end,
				set=function(v) self.db.account.showexmessages = v end,
			},
			autoscan= {
				type="toggle",
				name=L["Autoscan"],
				desc=L["Automatically scan your enchant panel when it opens"],
				get=function() return self.db.account.autoscan end,
				set=function(v) EnchantList:ToggleAutoScan(v) end,
			},
			desc= {
				type="toggle",
				name=L["Search description"],
				desc=L["Also search enchant description for a match"],
				get=function() return self.db.account.searchdesc end,
				set=function(v) self.db.account.searchdesc = v end,
			},
			scan= {
				type="execute",
				name=L["Manual scan (open your enchant panel first)"],
				desc=L["Manual scan (open your enchant panel first)"],
				func=function() EnchantList:Scan() end,
				order=101,
			},
			tlock= {
				type="range",
				name=L["Lock time between 2 requests (antispam)"],
				desc=L["Lock time between 2 requests (antispam)"],
				get=function() return self.db.account.TIMELOCK end,
				set=function(v) self.db.account.TIMELOCK = v end,
				min=1,
				max=30,
				step=1,
			},
			prefix= {
				type="text",
				name=L["Prefix"],
				desc=L["Set the prefix for your answers (have to be specific enough)"],
				usage="...",
				get=function() return self.db.account.chatprefix2 end,
				set=function(v) if v=='' then self:Print(L["Can't be empty"]) else self.db.account.chatprefix2 = v end end,
				order=102,
			},
			maxresults= {
				type="range",
				name=L["Maximum results"],
				desc=L["Maximum of results sent to someone (0=no limit)"],
				get=function() return self.db.account.maxresults end,
				set=function(v) self.db.account.maxresults = v end,
				min=0,
				max=100,
				step=1,
			},
			gmaxresults= {
				type="range",
				name=L["Maximum results for guild channel"],
				desc=L["Maximum of results for guild channel (0=no limit)"],
				get=function() return self.db.account.gmaxresults end,
				set=function(v) self.db.account.gmaxresults = v end,
				min=0,
				max=100,
				step=1,
			},
			
			listignored= {
				type="execute",
				name=L["List ignored enchants"],
				desc=L["List ignored enchants"],
				func=function() 
					if table.getn(self.db.account.ignoredenchants)==0 then self:Print(L["The ignore list is empty."]) return end
					self:Print(L["Ignored enchants:"])
					for i=1,table.getn(self.db.account.ignoredenchants) do self:Print(self.db.account.ignoredenchants[i]) end 
				end,
				order=105,
			},		
			
			purgeignored= {
				type="execute",
				name=L["Purge ignored enchants list"],
				desc=L["Purge ignored enchants list"],
				func=function() self.db.account.ignoredenchants={} self:Print(L["The ignore list is now empty."])end,
				order=106,
			},	
			guildactive= {
				type="toggle",
				name=L["Monitor guildchat"],
				desc=L["Monitor guildchat for !enchant requests"],
				get=function() return self.db.account.guildactive end,
				set=function(v) self.db.account.guildactive = v end,
			},			
		}
}

EnchantList:RegisterChatCommand({"/elist", "/enchantlist"}, options)
end

function EnchantList:OnEnable()
	self:Hook("ChatFrame_MessageEventHandler", "WhisperHandler", true)
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_GUILD")
	if self.db.account.autoscan then self:RegisterEvent("CRAFT_SHOW") end
	self:Hook("GetCraftInfo",true)
	
end

function EnchantList:OnDisable()
	self:Reset()
	self:UnhookAll()
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_GUILD")
	self:UnregisterEvent("CHAT_MSG_GUILD")
	self:RegisterEvent("CRAFT_SHOW")
	self:UnregisterEvent("CRAFT_SHOW")
end

function EnchantList:Reset()
	LOCKED=0
end

function EnchantList:GetCraftInfo(craftIndex)
	local craftName, craftSubSpellName, craftType, numAvailable, isExpanded, trainingPointCost, requiredLevel = self.hooks["GetCraftInfo"](craftIndex)
	local link = GetCraftItemLink(craftIndex)
	if not CraftFrameTitleText then return self.hooks["GetCraftInfo"](craftIndex) end
	if CraftFrameTitleText:GetText() ~= L["Enchanting"] then return self.hooks["GetCraftInfo"](craftIndex) end
	for k,v in pairs(self.db.account.ignoredenchants) do
			if v==link then 
				craftName="[x] "..craftName
			end
		end
	return craftName, craftSubSpellName, craftType, numAvailable, isExpanded, trainingPointCost, requiredLevel
end

function EnchantList:TradeClick(id)
	self.hooks["CraftFrame_SetSelection"](id)
	local index=GetCraftSelectionIndex()
	local craft = GetCraftItemLink(index)
	if IsControlKeyDown() then self:ToggleIgnore(craft) end
end

function EnchantList:ToggleIgnore(craftname)
		local found=false
		for k,v in pairs(self.db.account.ignoredenchants) do
			if v==craftname then 
				found=true
				table.remove(self.db.account.ignoredenchants,k)
				self:Print(craftname..L[" has been removed from the ignore list."])
			end
		end
		
		if found==false then
			 table.insert(self.db.account.ignoredenchants,craftname)
			 self:Print(craftname..L[" has been added to the ignore list."])
		end
end

function EnchantList:WhisperHandler(event)
	if arg1 and arg1~="" then
		if (string.sub(arg1,1,string.len(chatprefix))==chatprefix and not self.db.account.showincmessages and event=="CHAT_MSG_WHISPER") then
			return
		end
		if ( string.sub(arg1,1,string.len(self.db.account.chatprefix2))==self.db.account.chatprefix2 and not self.db.account.showexmessages and event=="CHAT_MSG_WHISPER_INFORM") then
			return
		end
	end
	return self.hooks["ChatFrame_MessageEventHandler"](event)
end

function EnchantList:CHAT_MSG_WHISPER()
	local argum = self:GetArgs(arg1," ")
	local sender = arg2
	if argum[1]==chatprefix then 
		if arg6==L["AFK"] or arg6==L["DND"] then self:Respond(L["First clear your tag: "]..arg6,sender) return end
		table.remove(argum,1)
		argum=self:RecomposeLink(argum)
		if table.getn(argum)==0 then 
			self:Respond(L["Usage:"],sender) 
			self:Respond(L["!enchant <text1> <text2> ... (ie. !enchant boots agi, !enchant crusader,... )"],sender) 
			self:Respond(L["!enchant <itemlink> (ie. !enchant [Ashbringer])"],sender) 
			self:Respond(L["!enchant <itemlink> <text> (ie. !enchant [Ashbringer] agi)"],sender) 
			if self.db.account.searchdesc then self:Respond(L["Searching enchant description is currently enabled."],sender)
			else self:Respond(L["Searching enchant description is currently disabled."],sender)
			end
			return 
		end
		for k,v in pairs(argum) do
			if strfind(argum[k],"|cff") and strfind(argum[k],"|h|r") then
				local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iel = GetItemInfo(argum[k])
				if sType == L["Armor"] then 
					local b=0
					if iel=="INVTYPE_CHEST" then b=L["Chest"]
					elseif iel=="INVTYPE_ROBE" then b=L["Chest"]
					elseif iel=="INVTYPE_FEET" then b=L["Boots"]
					elseif iel=="INVTYPE_WRIST" then b=L["Bracer"]
					elseif iel=="INVTYPE_HAND" then b=L["Gloves"]
					elseif iel=="INVTYPE_CLOAK" then b=L["Cloak"]
					elseif iel=="INVTYPE_SHIELD" then b=L["Shield"]
					elseif iel=="INVTYPE_FINGER" then b=L["Ring"]
					else self:Respond(L["Error! Cannot enchant that."],sender) return end
					argum[k]=b
				elseif sType==L["Weapon"] then 
					local b=0
					if iel=="INVTYPE_2HWEAPON" then b=L["Weapon"]
					elseif iel=="INVTYPE_WEAPONMAINHAND" then b=L["Enchant Weapon"]
					elseif iel=="INVTYPE_WEAPONOFFHAND" then b=L["Enchant Weapon"]
					elseif iel=="INVTYPE_WEAPON" then b=L["Enchant Weapon"]
					else self:Respond(L["Error! Cannot enchant that."],sender) return end
					argum[k]=b
				end
			end
		end
		if argum[1]=="next" then 
			if mem_search[sender]==nil then return end
			mem_next[sender]=mem_next[sender]+self.db.account.maxresults
			self:Handle(mem_search[sender],sender,mem_next[sender])
		else
			mem_search[sender]=argum 
			mem_next[sender]=0
			self:Handle(argum,sender,0)
		end
	end
end

function EnchantList:GetArgs(text, separator)
	local args = {}
	local i = 0
	for value in string.gmatch(text, "[^"..separator.."]+") do
		i = i + 1
		args[i] = value
	end	
	return args
end

function EnchantList:RecomposeLink(argument)
	local message=argument
	for k,v in pairs(message) do
		if strfind(v,"|cff") then
				if strfind(v,"|h|r") then return message
				else 
					message[k]=v.." "..message[k+1]
					table.remove(message,k+1)
					return self:RecomposeLink(message)
				end
		end
	end
	return message
end

function EnchantList:Handle(argum,sender,next)
	if table.getn(self.db.account.enchants)==0 then 
		self:Respond(L["Database is currently empty."],sender) 
		self:Print(sender..L[" wants to use EnchantList but the enchanting panel has not yet been opened."])
		return 
	end
	local searchresult={}
	for i=1,table.getn(self.db.account.enchants) do
		local paf=0
		local enchantname=string.lower(self.db.account.enchantsname[i])
		local enchantdesc=string.lower(self.db.account.enchantsdesc[i])
		for j=1,table.getn(argum) do
			if strfind(enchantname, string.lower(argum[j])) then paf=paf+1 
			elseif self.db.account.searchdesc and strfind(enchantdesc, string.lower(argum[j])) then paf=paf+1
			end
		end 
		local ignored=false
		for k,v in pairs(self.db.account.ignoredenchants) do 
			if v==self.db.account.enchants[i] then ignored=true end
		end
		if paf==table.getn(argum) and not ignored then table.insert(searchresult,self.db.account.enchants[i]) end	
	end
	if table.getn(searchresult)==0 then self:Respond(L["Nothing found."],sender)
	else 
		local max=table.getn(searchresult)
		if next > 0 then
			local i=0
			while i<next do
				table.remove(searchresult,1)
				i=i+1
			end
		end
		if LOCKED==0 then
			LOCKED=1
			if table.getn(searchresult) <= self.db.account.maxresults or self.db.account.maxresults == 0 then 
				for i=1,table.getn(searchresult) do 
					self:Respond(searchresult[i],sender) 
				end 
				self:Respond(table.getn(searchresult).." "..L["enchantment(s) found."],sender)
			else 
				for i=1,self.db.account.maxresults do 
					self:Respond(searchresult[i],sender) 
				end 
			self:Respond(L["Maximum results exceeded, be more specific."].." "..next.."-"..next+self.db.account.maxresults.."/"..max,sender)
			self:Respond(L["To get the next results, use !enchant next"],sender)
			end
			self:ScheduleEvent(function() LOCKED=0 return end, self.db.account.TIMELOCK)
		else 
			self:Respond(L["Wait ... ("]..self.db.account.TIMELOCK..L[" seconds lock (antispam))"],sender)
			if next~=0 then mem_next[sender]=mem_next[sender]-self.db.account.maxresults end
		end
	end
end

function EnchantList:Respond(message,sender)
	SendChatMessage(self.db.account.chatprefix2.." "..message, "WHISPER", nil, sender)
end

function EnchantList:CRAFT_SHOW()
	if self.db.account.autoscan then 
		if not CraftFrameTitleText then return end
		if CraftFrameTitleText:GetText() ~= L["Enchanting"] then return end
		if not self:IsHooked("CraftFrame_SetSelection") then self:Hook("CraftFrame_SetSelection", "TradeClick", true) end
		self:Scan()
	end
end

function EnchantList:ToggleAutoScan(v)
	self.db.account.autoscan = v 
	self:RegisterEvent("CRAFT_SHOW")
	if not v then self:UnregisterEvent("CRAFT_SHOW") end
end

function EnchantList:Scan()
	--if table.getn(self.enchants)~=0 then return end
	if not CraftFrameTitleText then self:Print(L["Open your enchant panel first"]) return end
	if CraftFrameTitleText:GetText() ~= L["Enchanting"] then self:Print(L["Open your enchant panel first"]) return end
	self.db.account.enchants={}
	self.db.account.enchantsname={}
	self.db.account.enchantsdesc={}
	for i=1,GetNumCrafts() do
		local skillname, craftSubSpellName, skilltype = GetCraftInfo(i)
		if skilltype~="header" and skillname then
				local link = GetCraftItemLink(i)
				local desc = GetCraftDescription(i)
				table.insert(self.db.account.enchants,link)
				table.insert(self.db.account.enchantsname,skillname)
			  if desc then table.insert(self.db.account.enchantsdesc,desc) else table.insert(self.db.account.enchantsdesc,"") end
		end	
	end
	return
end


function EnchantList:CHAT_MSG_GUILD()
	if not self.db.account.guildactive then return end
	local argum = self:GetArgs(arg1," ")
	if argum[1]==chatprefix then 
		table.remove(argum,1)
		argum=self:RecomposeLink(argum)
		if table.getn(argum)==0 then return end
		for k,v in pairs(argum) do
			if strfind(argum[k],"|cff") and strfind(argum[k],"|h|r") then
				local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iel = GetItemInfo(argum[k])
				if sType == L["Armor"] then 
					local b=0
					if iel=="INVTYPE_CHEST" then b=L["Chest"]
					elseif iel=="INVTYPE_ROBE" then b=L["Chest"]
					elseif iel=="INVTYPE_FEET" then b=L["Boots"]
					elseif iel=="INVTYPE_WRIST" then b=L["Bracer"]
					elseif iel=="INVTYPE_HAND" then b=L["Gloves"]
					elseif iel=="INVTYPE_CLOAK" then b=L["Cloak"]
					elseif iel=="INVTYPE_SHIELD" then b=L["Shield"]
					elseif iel=="INVTYPE_FINGER" then b=L["Ring"]
					else self:Respond(L["Error! Cannot enchant that."],sender) return end
					argum[k]=b
				elseif sType==L["Weapon"] then 
					local b=0
					if iel=="INVTYPE_2HWEAPON" then b=L["Weapon"]
					elseif iel=="INVTYPE_WEAPONMAINHAND" then b=L["Enchant Weapon"]
					elseif iel=="INVTYPE_WEAPONOFFHAND" then b=L["Enchant Weapon"]
					elseif iel=="INVTYPE_WEAPON" then b=L["Enchant Weapon"]
					else self:Respond(L["Error! Cannot enchant that."],sender) return end
					argum[k]=b
				end
			end
		end

			self:HandleGuild(argum)
		end
end


function EnchantList:HandleGuild(argum)

	if table.getn(self.db.account.enchants)==0 then 
		self:RespondGuild(L["Database is currently empty."]) 
		self:Print(sender..L[" wants to use EnchantList but the enchanting panel has not yet been opened."])
		return 
	end
	local searchresult={}
	for i=1,table.getn(self.db.account.enchants) do
		local paf=0
		local enchantname=string.lower(self.db.account.enchantsname[i])
		local enchantdesc=string.lower(self.db.account.enchantsdesc[i])
		for j=1,table.getn(argum) do
			if strfind(enchantname, string.lower(argum[j])) then paf=paf+1 
			elseif self.db.account.searchdesc and strfind(enchantdesc, string.lower(argum[j])) then paf=paf+1
			end
		end 
		local ignored=false
		for k,v in pairs(self.db.account.ignoredenchants) do 
			if v==self.db.account.enchants[i] then ignored=true end
		end
		if paf==table.getn(argum) and not ignored then table.insert(searchresult,self.db.account.enchants[i]) end	
	end
	if table.getn(searchresult)==0 then self:RespondGuild(L["Nothing found."])
	else 
		if LOCKED==0 then
			LOCKED=1
			if table.getn(searchresult) <= self.db.account.gmaxresults or self.db.account.gmaxresults == 0 then 
				for i=1,table.getn(searchresult) do 
					self:RespondGuild(searchresult[i]) 
				end 
			else 
				for i=1,self.db.account.gmaxresults do 
					self:RespondGuild(searchresult[i]) 
				end 
			self:RespondGuild(L["Maximum results exceeded, be more specific."])
			end
			self:ScheduleEvent(function() LOCKED=0 return end, self.db.account.TIMELOCK)
		end
	end
end

function EnchantList:RespondGuild(message)
	SendChatMessage(self.db.account.chatprefix2.." "..message, "GUILD", nil)
end
